package com.bib.essensbestellungsverwaltung;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.Spinner;
import javafx.scene.input.MouseEvent;
import javafx.scene.text.Text;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Calendar;
import java.util.List;

public class InvoiceController {
    @FXML
    Text responseText;
    @FXML
    ChoiceBox monatChoiceBox;

    @FXML
    ChoiceBox childChoiceBox;

    @FXML
    Spinner jahrSpinner;

    @FXML
    public void initialize() {
        monatChoiceBox.setValue(intToMonth(Calendar.getInstance().get(Calendar.MONTH) + 1));
        jahrSpinner.getValueFactory().setValue(Calendar.getInstance().get(Calendar.YEAR));

        List<Child> childList = AccountMgr.getAllChildren();
        ObservableList<Object> childOptions = FXCollections.observableArrayList(childList.stream()
                .map(c -> c.getId() + ": " + c.getFirstname() + " " + c.getName()).toList().toArray(new String[0]));
        childChoiceBox.setItems(childOptions);
    }

    @FXML
    void onRechnungErstellenClick(MouseEvent mouseEvent) {
        if (childChoiceBox.getValue() == null) {
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle("Es wurde kein Kind ausgewählt");
            alert.setHeaderText("Bitte wählen sie ein Kind aus");
            alert.showAndWait();
        }

        String childId = childChoiceBox.getValue().toString().split(":")[0];
        String date = String.format("%d-%02d", Integer.parseInt(jahrSpinner.getValue().toString()),
                monthToInt(monatChoiceBox.getValue().toString()));

        System.out.println("Invoice (" + date + ") from child: " + childId);

        List<String> invoice = AccountMgr.getInvoice(date, childId);

        responseText.setText(invoice.get(invoice.size() - 1));

        // TODO: show invoice
        Child child = AccountMgr.getChildById(Long.parseLong(childId));
        exportInvoice("Rechnung_" + date + "_" + childId + "_" + child.getName() +"_" + child.getFirstname(), invoice);
    }

    private int monthToInt(String month) {
        return switch (month) {
            case "Januar" -> 1;
            case "Februar" -> 2;
            case "März" -> 3;
            case "April" -> 4;
            case "Mai" -> 5;
            case "Juni" -> 6;
            case "Juli" -> 7;
            case "August" -> 8;
            case "September" -> 9;
            case "Oktober" -> 10;
            case "November" -> 11;
            case "Dezember" -> 12;
            default -> -1;
        };
    }

    private String intToMonth(int month) {
        return switch (month) {
            case 1 -> "Januar";
            case 2 -> "Februar";
            case 3 -> "März";
            case 4 -> "April";
            case 5 -> "Mai";
            case 6 -> "Juni";
            case 7 -> "Juli";
            case 8 -> "August";
            case 9 -> "September";
            case 10 -> "Oktober";
            case 11 -> "November";
            case 12 -> "Dezember";
            default -> "";
        };
    }

    private void exportInvoice(String filename, List<String> invoice) {
        try {
            Files.createDirectories(Path.of(Path.of("").toAbsolutePath() + "/Rechnungen"));

            // TODO: save invoice to pdf or word
            File file = new File(Path.of("").toAbsolutePath() + "/Rechnungen/" + filename + ".txt");
            if (file.createNewFile()) {
                System.out.println("File created: " + file.getName());
            } else {
                System.out.println("File already exists.");
            }
            Files.write(Path.of(file.getAbsolutePath()), invoice);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
